/***************************************************************************
 *   Copyright (c) 2023 Ondsel, Inc.                                       *
 *                                                                         *
 *   This file is part of OndselSolver.                                    *
 *                                                                         *
 *   See LICENSE file for details about copyright.                         *
 ***************************************************************************/
 
#include "CompoundJoint.h"

using namespace MbD;

MbD::CompoundJoint::CompoundJoint()
{
}

MbD::CompoundJoint::CompoundJoint(const std::string& str) : Joint(str)
{
}
