/***************************************************************************
 *   Copyright (c) 2023 Ondsel, Inc.                                       *
 *                                                                         *
 *   This file is part of OndselSolver.                                    *
 *                                                                         *
 *   See LICENSE file for details about copyright.                         *
 ***************************************************************************/
 
#pragma once

#include "DistxyIeqcJeqc.h"

namespace MbD {
	class DistxyIeqctJeqc :
		public DistxyIeqcJeqc
	{
		//pdistxypt ppdistxypXIpt ppdistxypEIpt ppdistxypXJpt ppdistxypEJpt ppdistxyptpt 
	public:
		//ToDo: add member functions.

	};
}

