/***************************************************************************
 *   Copyright (c) 2023 Ondsel, Inc.                                       *
 *                                                                         *
 *   This file is part of OndselSolver.                                    *
 *                                                                         *
 *   See LICENSE file for details about copyright.                         *
 ***************************************************************************/
 
#include "FunctionFromData.h"

using namespace MbD;

MbD::FunctionFromData::FunctionFromData(Symsptr arg) : FunctionXcParameter(arg)
{
}
